import { system } from '@minecraft/server';
import { Command } from '../handler';
import { sendPlayerMessage, config } from '../../config';
import { lifesteal, getPlayer, getScore } from '../../lifesteal';
export function heartHelp(player) {
    if (player.hasTag(config.staffTag) || player.isOp()) {
        sendPlayerMessage(player, `§e§lLifesteal Help Page -- Heart§r§6
> Withdraw a certain amount of hearts into your inventory

> Usage: ${config.prefix}heart <display/show/set/reset/setmax>

> Args: [
    display - Display your max health above your head
    show - Display your max health to only yourself
    set - Set a player's max health
    setmax - Set the maximum amount of hearts
    envsteal - Toggle environment steal (whether you lose hearts from natural causes like mobs)
    banghosts - Option to ban ghosts
    heartgl - Set the amount of hearts you gain or lose when you kill a player or die 
    add - Add to a player's heart count
    remove - Remove hearts from a player's heart count
    get - See how many hearts a player has
]

> Examples: [
    ${config.prefix}heart display on
    ${config.prefix}heart display off
    ${config.prefix}heart show on
    ${config.prefix}heart show off
    ${config.prefix}heart set 16 ${player.name}
    ${config.prefix}heart set 16
    ${config.prefix}heart setmax 30
    ${config.prefix}heart envsteal true
    ${config.prefix}heart envsteal false
    ${config.prefix}heart banghosts true
    ${config.prefix}heart banghosts false
    ${config.prefix}heart heartgl 4
    ${config.prefix}heart add 5
    ${config.prefix}heart remove 5
    ${config.prefix}heart get
    ${config.prefix}heart get ${player.name}
]`);
        return;
    }
    sendPlayerMessage(player, `§e§lLifesteal Help Page -- Heart§r§6
> Withdraw a certain amount of hearts into your inventory

> Usage: ${config.prefix}heart <display/show/set/setmax>

> Args: [
display - Display your max health above your head
show - Display your max health to only yourself
]

> Examples: [
${config.prefix}heart display on
${config.prefix}heart display off
${config.prefix}heart show on
${config.prefix}heart show off
]`);
}
function staffCheck(player) {
    if (!player.hasTag(config.staffTag)) {
        if (player.isOp()) { }
        else
            return false;
    }
}
new Command({
    name: 'heart',
    code({ player, args }) {
        if (!args[0])
            return heartHelp(player);
        switch (args[0]) {
            default:
                sendPlayerMessage(player, `§cInvalid argument(s), do ?help heart for help on this command`);
                break;
            case `add`:
                function addHearts() {
                    if (staffCheck(player) == false)
                        return sendPlayerMessage(player, `§cYou do not have permission to run this command! Do /tag @s add lifesteal-opped to get permission.`);
                    const amount = +args[1];
                    if (!amount)
                        return sendPlayerMessage(player, `§cAn argument for amount of hearts was not provided`);
                    if (isNaN(amount))
                        return sendPlayerMessage(player, `§cThat is not a number!`);
                    let target;
                    if (args.slice(2, args.length).join(` `)) {
                        target = getPlayer(args.slice(2, args.length).join(` `));
                        if (!target)
                            return sendPlayerMessage(player, `§cFailed to find player, player must be online`);
                    }
                    else
                        target = player;
                    target.runCommandAsync(`scoreboard players add @s ls-death ${amount}`);
                    system.runTimeout(() => {
                        sendPlayerMessage(player, `§aAdded §f${amount}§a hearts to §f${target.name}'s§a heart count §f(Now ${getScore(target, `ls-death`)})`);
                    }, 1);
                    if (target.hasTag(`ls-dead`)) {
                        target.runCommandAsync(`tag @s remove ls-dead`);
                        target.runCommandAsync(`gamemode s`);
                    }
                }
                addHearts();
                break;
            case `remove`:
                function removeHearts() {
                    if (staffCheck(player) == false)
                        return sendPlayerMessage(player, `§cYou do not have permission to run this command! Do /tag @s add lifesteal-opped to get permission.`);
                    const amount = +args[1];
                    if (!amount)
                        return sendPlayerMessage(player, `§cAn argument for amount of hearts was not provided`);
                    if (isNaN(amount))
                        return sendPlayerMessage(player, `§cThat is not a number!`);
                    let target;
                    if (args.slice(2, args.length).join(` `)) {
                        target = getPlayer(args.slice(2, args.length).join(` `));
                        if (!target)
                            return sendPlayerMessage(player, `§cFailed to find player, player must be online`);
                    }
                    else
                        target = player;
                    target.runCommandAsync(`scoreboard players remove @s ls-death ${amount}`);
                    system.runTimeout(() => {
                        sendPlayerMessage(player, `§aRemoved §f${amount}§a hearts to §f${target.name}'s§a heart count §f(Now ${getScore(target, `ls-death`)})`);
                    }, 1);
                }
                removeHearts();
                break;
            case `get`:
                function getHearts() {
                    if (staffCheck(player) == false)
                        return sendPlayerMessage(player, `§cYou do not have permission to run this command! Do /tag @s add lifesteal-opped to get permission.`);
                    let target;
                    if (args.slice(1, args.length).join(` `)) {
                        target = getPlayer(args.slice(1, args.length).join(` `));
                        if (!target)
                            return sendPlayerMessage(player, `§cFailed to find player, player must be online`);
                    }
                    else
                        target = player;
                    sendPlayerMessage(player, `§f${target.name}§a has §f${getScore(target, `ls-death`)}§a hearts`);
                }
                getHearts();
                break;
            case `banghosts`:
                if (staffCheck(player) == false)
                    return sendPlayerMessage(player, `§cYou do not have permission to run this command! Do /tag @s add lifesteal-opped to get permission.`);
                switch (args[1]) {
                    default:
                        sendPlayerMessage(player, `§cInvalid argument(s) detected, must be true or false`);
                        break;
                    case `true`:
                        lifesteal.set(`banghosts`, `true`);
                        sendPlayerMessage(player, `§aSet §fbanGhosts§a to §ftrue`);
                        break;
                    case `false`:
                        lifesteal.set(`banghosts`, `false`);
                        sendPlayerMessage(player, `§aSet §fbanGhosts§a to §ffalse`);
                        break;
                }
                break;
            case `heartgl`:
                if (staffCheck(player) == false)
                    return sendPlayerMessage(player, `§cYou do not have permission to run this command! Do /tag @s add lifesteal-opped to get permission.`);
                if (isNaN(+args[1]))
                    return sendPlayerMessage(player, `§cThats not a number!`);
                if (+args[1] == 0 || +args[1] > 4)
                    return sendPlayerMessage(player, `§cYou can only set HeartGL to 1-4`);
                lifesteal.set(`heartgl`, `${args[1]}`);
                sendPlayerMessage(player, `§aSet §fheartGL §ato §f${args[1]}`);
                break;
            case `display`:
                switch (args[1]) {
                    default:
                        sendPlayerMessage(player, `§cInvalid arguments detected, do ${config.prefix}help heart for a list of arguments`);
                        break;
                    case `on`:
                        if (player.hasTag(`heart-display_on`))
                            return sendPlayerMessage(player, `§cHeart display (public) is already on!`);
                        player.addTag(`heart-display_on`);
                        sendPlayerMessage(player, `§6Heart display (public) turned on (Might be glitchy)`);
                        break;
                    case `off`:
                        if (!player.hasTag(`heart-display_on`))
                            return sendPlayerMessage(player, `§cHeart display (public) is already off!`);
                        player.removeTag(`heart-display_on`);
                        sendPlayerMessage(player, `§6Heart display (public) turned off`);
                        break;
                }
                break;
            case `show`:
                switch (args[1]) {
                    default:
                        sendPlayerMessage(player, `§cInvalid arguments detected, do ${config.prefix}help heart for a list of arguments`);
                        break;
                    case 'on':
                        if (player.hasTag(`heart-show_on`))
                            return sendPlayerMessage(player, `§cHeart display (private) is already on!`);
                        player.addTag(`heart-show_on`);
                        sendPlayerMessage(player, `§6Heart display (private) turned on (Might be glitchy)`);
                        break;
                    case 'off':
                        if (!player.hasTag(`heart-show_on`))
                            return sendPlayerMessage(player, `§cHeart display (private) is already off!`);
                        player.removeTag(`heart-show_on`);
                        sendPlayerMessage(player, `§6Heart display (private) turned off`);
                        break;
                }
                break;
            case 'set':
                if (staffCheck(player) == false)
                    return sendPlayerMessage(player, `§cYou do not have permission to run this command! Do /tag @s add lifesteal-opped to get permission.`);
                let amount = args[1];
                if (!amount || isNaN(+amount))
                    return sendPlayerMessage(player, `§cInvalid amount entered`);
                if (!args[2]) {
                    if (!amount)
                        return sendPlayerMessage(player, `§cInvalid amount entered`);
                    if (player.hasTag(`ls-dead`)) {
                        player.runCommandAsync(`tag @s remove ls-dead`);
                        player.runCommandAsync(`scoreboard players set @s ls-death ${amount}`);
                        player.runCommandAsync(`gamemode s`);
                    }
                    player.runCommandAsync(`scoreboard players set @s ls-death ${amount}`);
                    sendPlayerMessage(player, `§aSuccessfully set §f${player.name}'s§a hearts to §f${amount}`);
                    return;
                }
                let target = getPlayer(args.slice(2, args.length).join(` `));
                if (!target)
                    return sendPlayerMessage(player, `§cFailed to find target`);
                if (target.hasTag(`ls-dead`)) {
                    target.runCommandAsync(`tag @s remove ls-dead`);
                    target.runCommandAsync(`scoreboard players set @s ls-death ${amount}`);
                    target.runCommandAsync(`gamemode s`);
                }
                target.runCommandAsync(`scoreboard players set @s ls-death ${amount}`);
                sendPlayerMessage(player, `§aSuccessfully set §f${target.name}'s§a hearts to §f${amount}`);
                break;
            case 'setmax':
                if (staffCheck(player) == false)
                    return sendPlayerMessage(player, `§cYou do not have permission to run this command! Do /tag @s add lifesteal-opped to get permission.`);
                if (!args[1] || isNaN(+args[1]))
                    return sendPlayerMessage(player, `§cThat isn't a number!`);
                if (+args[1] > 100)
                    return sendPlayerMessage(player, `§cYou can't set the max hearts to over 100!`);
                if (+args[1] < 10)
                    return sendPlayerMessage(player, `§cYou can't set the max hearts to less than 10!`);
                lifesteal.set(`maxhearts`, `${args[1]}`);
                sendPlayerMessage(player, `§aSet maximum hearts to §f${args[1]}`);
                break;
            case 'envsteal':
                if (staffCheck(player) == false)
                    return sendPlayerMessage(player, `§cYou do not have permission to run this command! Do /tag @s add lifesteal-opped to get permission.`);
                let acceptable = [`true`, `false`];
                if (!args[1] || !acceptable.includes(args[1].toLowerCase()))
                    return sendPlayerMessage(player, `§cArgument must be true or false`);
                lifesteal.set(`envsteal`, `${args[1]}`);
                sendPlayerMessage(player, `§aSet environment steal to §f${args[1]}`);
                break;
        }
    }
});
