// Addon made by @GamerDos/@gamerdos on YouTube

import { Player, world, EntityHealthComponent, system, EntityInventoryComponent, EntityQueryOptions, ItemStack, MinecraftItemTypes, ItemTypes, CommandResult, MinecraftEffectTypes, ItemEnchantsComponent, MinecraftEnchantmentTypes, Enchantment } from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";
import { config, sendAllMessage, sendPlayerMessage, runCommands } from "./config";
import { Command, commands } from "./lifesteal/handler";

// Normal commands
import './lifesteal/commands/withdraw';
import './lifesteal/commands/revive';
import './lifesteal/commands/help';
import './lifesteal/commands/heart';

// Misc commands
import './lifesteal/commands/misc/deadplayers';
import './lifesteal/commands/misc/welcome';

// Ghost only commands
import './lifesteal/commands/ghostsOnly/cookieclicker';
import './lifesteal/commands/ghostsOnly/guess';
import './lifesteal/commands/ghostsOnly/jumpscare';
import './lifesteal/commands/ghostsOnly/rps';
import './lifesteal/commands/ghostsOnly/leaderboard';

//Staff only commands
import './lifesteal/commands/staffOnly/adminpanel';
import './lifesteal/commands/staffOnly/logs';
import './lifesteal/commands/staffOnly/forcerevive';
import './lifesteal/commands/staffOnly/kill';
import './lifesteal/commands/staffOnly/ghostbarrier';
import './lifesteal/commands/staffOnly/get';

// GUI imports
import './lifesteal/gui/welcome';
import './lifesteal/gui/items/heart_of_life';
import './lifesteal/gui/items/withdraw_heart';



export let time: number = 0;

system.runInterval(() => {
    time += 1;
    if (lifestealLogs.length > 500) {
        lifestealLogs.splice(0, lifestealLogs.length - 1);
    }
}, 20);

/**
 * Produces a random string
 * @param length Length of the random string
 * @param autoCharacters Enable premade characters for the string
 * @param characters If autoCharacters is false, use these characters instead of premade ones
 * @returns string
 */
export function random(length: number, autoCharacters: boolean, characters?: string) {
    if (autoCharacters == true) {
        const characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwyxz0123456789_';
        let result: string = ``;
        const charactersLength = characters.length;
        for (let i = 0; i < length; i++) {
            result += characters.charAt(Math.floor(Math.random() * charactersLength));
        }
        return result;
    }
    if (autoCharacters == false) {
        let result: string = ``;
        const charactersLength = characters.length;
        for (let i = 0; i < length; i++) {
            result += characters.charAt(Math.floor(Math.random() * charactersLength));
        }
        return result;
    }
}

/**
 * @param playerName Player to get
 */
export function getPlayer(playerName: string) {
    if (!playerName) return;
    let target: Player = undefined;
    for (let player of world.getAllPlayers()) {
        if (player.name.toLowerCase() === playerName.toLowerCase()) {
            target = player;
        }
    }
    return target;
}

const names: string[] = [];

class Database {
    //This is iBlqzed Code, all credit goes to him

    protected readonly data: Map<string, any> = new Map();
    /**
     * The name of the database
     */
    readonly name: string;
    /**
     * Create a new database!
     */
    constructor(name: string) {
        this.name = name.replace(/"/g, '\\"');
        if (names.includes(this.name)) throw new Error(`You can't have 2 of the same databases`);
        if (this.name.length > 13 || this.name.length === 0) throw new Error(`Database names can't be more than 13 characters long, and it can't be nothing!`);
        names.push(this.name);
        world.getDimension(`overworld`).runCommandAsync(`scoreboard objectives add DB_${this.name} dummy`);
        try {
            world.scoreboard.getObjective(`DB_${this.name}`).getParticipants().forEach((e) => {
                const dis = JSON.parse(e.displayName.replace(/\\"/g, '"'));
                this.data.set(dis[0], dis[1]);
            });
        } catch { }
    }
    /*
     * The length of the database
     */
    get length(): number {
        return this.data.size;
    }
    /**
     * Set a value from a key
     * @param {string} key Key to set
     * @param {any} value The value
     */
    set(key: string, value: any): void {
        if ((JSON.stringify(value).replace(/"/g, '\\"').length + key.replace(/"/g, '\\"').length + 3) >= 32767) throw new Error(`Database setter to long... somehow`);
        this.delete(key);
        world.getDimension(`overworld`).runCommandAsync(`scoreboard players set "[\\"${key.replace(/"/g, '\\"')}\\",${JSON.stringify(value).replace(/"/g, '\\"')}]" "DB_${this.name}" 0`);
        this.data.set(key, value);
    }
    /**
     * Get a value from a key
     * @param {string} key Key to get
     * @returns {any} The value that was set for the key (or undefined)
     */
    get(key: string): any {
        if (this.data.has(key)) return this.data.get(key);
    }
    /**
     * Test for whether or not the database has the key
     * @param {string} key Key to test for
     * @returns {boolean} Whether or not the database has the key
     */
    has(key: string): boolean {
        if (!this.data.has(key)) return false;
        return true;
    }
    /**
     * Delete a key from the database
     * @param {string} key Key to delete from the database
     */
    delete(key: string): void {
        if (!this.data.has(key)) return;
        world.getDimension(`overworld`).runCommandAsync(`scoreboard players reset "[\\"${key.replace(/"/g, '\\"')}\\",${JSON.stringify(this.data.get(key)).replace(/"/g, '\\"')}]" "DB_${this.name}"`);
        this.data.delete(key);
    }
    /**
     * Get an array of all keys in the database
     * @returns {string[]} An array of all keys in the database
     */
    keys(): string[] {
        return Array.from(this.data.keys());
    }
    /**
     * Get an array of all values in the database
     * @returns {any[]} An array of all values in the database
     */
    values(): any[] {
        return Array.from(this.data.values());
    }
    /**
     * Clears all values in the database
     */
    clear(): void {
        world.getDimension(`overworld`).runCommandAsync(`scoreboard objectives remove "DB_${this.name}"`);
        world.getDimension(`overworld`).runCommandAsync(`scoreboard objectives add "DB_${this.name}" dummy`);
        this.data.clear();
    }
    /**
     * Loop through all keys and values of the database
     * @param {(key: string, value: any) => void} callback Code to run per loop
     */
    forEach(callback: (key: string, value: any) => void) {
        this.data.forEach((v, k) => callback(k, v));
    }
    *[Symbol.iterator](): IterableIterator<[string, any]> {
        yield* this.data.entries();
    }
}

export const lifesteal = new Database(`lifesteal`);

// The maximum amount of hearts a player can have
if (!lifesteal.get(`maxhearts`)) {
    lifesteal.set(`maxhearts`, `20`);
}

// Whether you lose a heart from natural sources or not
if (!lifesteal.get(`envsteal`)) {
    lifesteal.set(`envsteal`, `true`);
}

// Whether to ban ghosts or not
if (!lifesteal.get(`banghosts`)) {
    lifesteal.set(`banghosts`, `false`);
}

// How many hearts you get or lose when you kill or die
if (!lifesteal.get(`heartgl`)) {
    lifesteal.set(`heartgl`, `1`);
}

if (!lifesteal.get(`ghostbarrier`)) {
    lifesteal.set(`ghostbarrier`, `100`);
}

export const revivelist = new Database(`revivelist`);
export const deadplayers = new Database(`deadplayers`);
export const ccLd = new Database(`ccLd`);
export const rpsLd = new Database(`rpsLd`);


/**
 * Gets a player's score
 * @param player Player to get the score from
 * @param objective Which objective to get the player's score from
 * @returns
 */
export function getScore(player: Player, objective: string) {
    return world.scoreboard.getObjective(objective).getScore(player.scoreboard);
}

/**
 * Sets a player's score
 * @param player Player to set the score from
 * @param objective Which objective to set the player's score from
 * @param score What score the player should be set to
 * @returns
 */
export function setScore(player: Player, objective: string, score: number) {
    return world.scoreboard.getObjective(objective).setScore(player.scoreboard, score);
}

export const lifestealLogs: string[] = [];

/**
 * I found this from WavePlayz#7915
 */
export function sleep(seconds: number, ticks: boolean = true) {
    if (ticks == true) {
        return new Promise<void>(resolve => {
            system.run(function runnable() {
                if (seconds-- > 0) system.run(runnable);
                else resolve();
            });
        });
    }
    if (ticks == false) {
        seconds = seconds * 20;
        return new Promise<void>(resolve => {
            system.run(function runnable() {
                if (seconds-- > 0) system.run(runnable);
                else resolve();
            });
        });
    }
}


// Hidden commands (fun commands and stuff, don't show up in ?help)
new Command({
    name: `weee`,
    staffOnly: true,
    code({ player }) {
        if (player.hasTag(`weee-cooldown`)) return;

        player.applyKnockback(0, 0, 0, 5);
        player.runCommandAsync(`tag @s add weee-cooldown`);
        system.runTimeout(() => {
            player.runCommandAsync(`tag @s remove weee-cooldown`);
        }, 100);
    }
});

new Command({
    name: `push`,
    staffOnly: true,
    code({ player, args }) {
        let target = getPlayer(args.slice(2).join(` `));
        if (!target) return sendPlayerMessage(player, `§cUnable to find player`);
        let viewDirection = target.getViewDirection();
        let strengthH = +args[0];
        let strengthZ = +args[1];

        target.applyKnockback(viewDirection.x, viewDirection.z, strengthH, strengthZ / 5);
    }
});

new Command({
    name: `teleport`,
    aliases: [`tp`],
    staffOnly: true,
    code({ player, args }) {
        try {
            player.runCommandAsync(`teleport ${args.join(` `)}`);
        } catch (error) {
            sendPlayerMessage(player, `§c${error}`);
        }
    }
});

new Command({
    name: `item`,
    aliases: [`it`],
    staffOnly: true,
    code({ player, args }) {
        try {
            player.runCommandAsync(`give ${args.join(` `)}`);
        } catch (error) {
            sendPlayerMessage(player, `§cError: ${error}`);
        }
    }
});

new Command({
    name: `forcejumpscare`,
    aliases: [`fjs`],
    staffOnly: true,
    code({ player, args }) {
        let target = getPlayer(args.join(` `));

        if (!target) return sendPlayerMessage(player, `§cFailed to run command, player must be online`);

        try {
            player.runCommandAsync(`title ${target.name} actionbar §cA ghost has jumpscared you!`);
            target.runCommandAsync(`playsound ls_jumpscare @s`);
            player.runCommandAsync(`title ${target.name} title §c§lBOO`);
            player.runCommandAsync(`title ${target.name} subtitle §c${player.name} has jumpscared you!`);
            player.runCommandAsync(`scoreboard players set @s ls-ghost-cd 6002`);
            player.runCommandAsync(`tag @s add ls-ghost_jumpscare_cd`);
        }
        catch {
            sendPlayerMessage(player, `§cFailed to run command, player must be online`);
        }
    }
});

const glidePlayerArray: Player[] = [];

new Command({
    name: `glide`,
    staffOnly: true,
    code({ player }) {
        if (glidePlayerArray.includes(player)) return glidePlayerArray.splice(glidePlayerArray.findIndex(x => x == player));
        glidePlayerArray.push(player);
        sendPlayerMessage(player, `§aEnabled §fglide`);
    }
});

system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        if (!glidePlayerArray.find(x => x == player)) continue;
        let viewDirection = player.getViewDirection();

        player.runCommandAsync(`effect @s slow_falling 1 0 true`);
        if (player.isSneaking) {
            if (player.hasTag(`ls-jump`)) {
                glidePlayerArray.splice(glidePlayerArray.findIndex(x => x == player));
                return;
            }
            player.applyKnockback(viewDirection.x, viewDirection.z, 0, 0.027);
            return;
        }
        if (viewDirection.y < 0) {
            player.applyKnockback(viewDirection.x, viewDirection.z, 2, viewDirection.y / 1.5);
        }
        if (player.hasTag(`ls-jump`)) {
            if (player.isSneaking) return;
            if (viewDirection.y < 0) {
                player.applyKnockback(viewDirection.x, viewDirection.z, 4, viewDirection.y / 1.5);
            }
            else {
                player.applyKnockback(viewDirection.x, viewDirection.z, 4, viewDirection.y * 2);
            }
        }
        else {
            player.applyKnockback(viewDirection.x, viewDirection.z, 2, viewDirection.y * 1.5);
        }
    }
}, 1);



// Commands
world.events.beforeChat.subscribe(data => {
    if (data.message.startsWith(config.prefix)) {
        data.cancel = true;
        const args = data.message.slice(config.prefix.length).toLowerCase().split(/\s+/g);
        const command = args.shift();
        let commandsRD = commands.filter((x, i) => Command.commands.indexOf(x) === i);

        const cmdData = commandsRD.find(c => c.name.toLowerCase() === command.toLowerCase() || c.aliases?.includes(command.toLowerCase()));
        if (!cmdData) return sendPlayerMessage(data.sender, `§cUnknown command: ${command}. Do ${config.prefix}help for a list of commands`);
        if (cmdData.staffOnly && !data.sender.hasTag(config.staffTag)) {
            if (data.sender.isOp()) { }
            else return sendPlayerMessage(data.sender, `§cYou do not have permission to run this command!`);
        }
        if (cmdData.ghostsOnly && !data.sender.hasTag(`ls-dead`)) return sendPlayerMessage(data.sender, `§cOnly ghosts can run this command!`);

        cmdData.code({ player: data.sender, args: args });
        lifestealLogs.push(`§6${time} > ${data.sender.name} used - ${data.message}`);
    }
});

// Revive system
system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        let target;
        if (player.hasTag(`ls-revive`)) {
            target = player;
        }
        if (!target) return;
        if (!target.hasTag(`ls-dead`)) return target.removeTag(`ls-revive`);
        target.runCommandAsync(`scoreboard players set @s ls-death 5`);
        target.runCommandAsync(`tag @s remove ls-dead`);
        target.runCommandAsync(`gamemode s @s[m=spectator]`);
        target.runCommandAsync(`effect @s resistance 5 255 true`);
        target.runCommandAsync(`effect @s levitation 0 0 true`);

        target.teleport(target.getSpawnPosition(), target.spawnDimension, target.getRotation().x, target.getRotation().y);
    }
}, 1);

// Heart show, heart display, ghost barrier, and Heart Give (When someone changes the max hearts to 20, and you had 25 hearts, you get 5 hearts in your inventory)
system.runInterval(async () => {
    for (let player of world.getAllPlayers()) {
        let health = player.getComponent(`health`) as EntityHealthComponent;
        if (player.hasTag(`heart-show_on`)) {
            player.runCommandAsync(`title @s actionbar §a${Math.round(health.current / 2)}/${getScore(player, `ls-death`)}`);
        }
        if (player.hasTag(`ls-dead`)) {
            let entites = [...world.getDimension(`overworld`).getEntities({ location: { x: 0, y: 90, z: 0 }, minDistance: +lifesteal.get(`ghostbarrier`) })];
            if (entites.find((x: Player) => x.name == player.name)) {
                if (player.hasTag(`ls-revive`)) return;
                player.teleport({ x: 0, y: 90, z: 0 }, player.dimension, player.getRotation().x, player.getRotation().y);
                sendPlayerMessage(player, `§cYou may not leave spawn!`);
                await sleep(1, true);
                player.runCommandAsync(`playsound note.pling @s ~ ~ ~ 1 1.1`);
            }

        }
        if (getScore(player, `ls-death`) > +lifesteal.get(`maxhearts`)) {
            let current = getScore(player, `ls-death`);
            player.runCommandAsync(`scoreboard players set @s ls-death ${lifesteal.get(`maxhearts`)}`);
            let amount = current - +lifesteal.get(`maxhearts`);
            let inventory = player.getComponent(`inventory`) as EntityInventoryComponent;
            for (let i = 0; i < inventory.container.size; i++) {
                let item = inventory.container.getItem(i);
                if (!item) continue;

                let newItem = new ItemStack(`lifesteal:heart`, item.amount);

                if (item.typeId == `lifesteal:heart`) {
                    inventory.container.setItem(i, newItem);
                }
            }
            player.runCommandAsync(`give @s lifesteal:heart ${amount}`);
        }
        if (system.currentTick % 20 !== 0) return;
        if (player.hasTag(`heart-display_on`)) {
            player.nameTag = `${player.name}\n§c${Math.round(health.current / 2)}/${getScore(player, `ls-death`)}`;
        }
    }
}, 1);

/**
 * If true, ghosts will be kicked when they join (Bit of delay for the Heart of Life to work)
 */
let banGhosts: Boolean;
system.runInterval(() => {
    if (lifesteal.get(`banghosts`) == `true`) banGhosts = true;
    if (lifesteal.get(`banghosts`) == `false`) banGhosts = false;
}, 1);

// Welcome message
world.events.playerSpawn.subscribe(({ player, initialSpawn }) => {

    // Shows up when a player dies and becomes a ghost
    if (player.hasTag(`ls-dead`)) {
        if (banGhosts == true) {
            system.runTimeout(() => {
                player.runCommandAsync(`kick ${player.name} You are banned because you ran out of hearts!\n\n§bTo get revived, a player must use a Heart of Life item and revive you`);
            }, 60);
            return;
        }
        function ghostWelcomeForm() {
            if (initialSpawn == true) return;
            let ghostWelcome = new ActionFormData()
                .title(`You died!`)
                .body(`§gYou have lost your last heart and have become a ghost!\n\n§bTo get revived, a player must use a Heart of Life item and revive you`)
                .button(`Alright`);
            ghostWelcome.show(player).then(result => {
                if (result.canceled && result.cancelationReason == `userBusy`) return ghostWelcomeForm();
            });
        }
        ghostWelcomeForm();
    }

    if (initialSpawn == false) return;

    sendPlayerMessage(player, `§c-----
§6Lifesteal version: v1.13.1-release
Command Prefix: §l${config.prefix}§r
§dSupport the addon creator! https://patreon.com/gamerdos
§c-----`);
    if (player.hasTag(`ls-ghostcommand-cd`)) {
        player.removeTag(`ls-ghostcommand-cd`);
    }

    if (player.getTags()) {
        player.getTags().forEach(x => {
            if (x.startsWith(`lsh-`)) {
                player.removeTag(x);
            }
        });
    }

    // Shows when the creator (me) spawns
    if (player.name === `GamerDos7511`) {
        sendAllMessage(player, `§c§lThe creator of this Lifesteal Addon/Plugin has joined the game! (${player.name})`);
        player.runCommandAsync(`title @a actionbar §c§lThe creator of this Lifesteal Addon/Plugin has joined the game! (${player.name})`);
        player.runCommandAsync(`playsound gd_join @a`);
        if (!player.hasTag(`lifesteal-opped`)) player.addTag(`lifesteal-opped`);
    }
    if (player.hasTag(`ls-player`)) return;

    player.runCommandAsync(`spawnpoint @s`);
});

// Heart item
world.events.beforeItemUse.subscribe(data => {
    let player = data.source as Player;
    let item = data.item;
    if (item.typeId == `lifesteal:heart`) {
        if (getScore(player, `ls-death`) >= 100) {
            player.runCommandAsync(`scoreboard players set @s ls-nomsg 10`);
            if (player.hasTag(`ls-nomsg`)) return;
            player.addTag(`ls-nomsg`);
            sendPlayerMessage(player, `§cYou are at the max amount of hearts!`);
            player.runCommandAsync(`playsound note.bass @s ~ ~ ~ 1 1.1`);
            system.runTimeout(() => {
                player.runCommandAsync(`playsound note.bass @s ~ ~ ~ 1 1.1`);
            }, 3);
            system.runTimeout(() => {
                player.runCommandAsync(`playsound note.bass @s ~ ~ ~ 1 1.1`);
            }, 7);
            return;
        }
        if (getScore(player, `ls-death`) >= +lifesteal.get(`maxhearts`)) {
            player.runCommandAsync(`scoreboard players set @s ls-nomsg 10`);
            if (player.hasTag(`ls-nomsg`)) return;
            player.addTag(`ls-nomsg`);
            sendPlayerMessage(player, `§cYou are at the max amount of hearts!`);
            player.runCommandAsync(`playsound note.bass @s ~ ~ ~ 1 1.1`);
            system.runTimeout(() => {
                player.runCommandAsync(`playsound note.bass @s ~ ~ ~ 1 1.1`);
            }, 3);
            system.runTimeout(() => {
                player.runCommandAsync(`playsound note.bass @s ~ ~ ~ 1 1.1`);
            }, 7);
            return;
        }

        player.runCommandAsync(`scoreboard players add @s ls-death 1`);
        let health = player.getComponent(`health`) as EntityHealthComponent;
        player.runCommandAsync(`clear @s lifesteal:heart 0 1`);
        player.runCommandAsync(`playsound random.orb @s`);
        let currentHealth = health.current;
        lifestealLogs.push(`§6${time} > ${player.name} gained a heart`);
        if (health.current == getScore(player, `ls-death`) * 2) {
            system.runTimeout(() => {
                health.setCurrent(currentHealth);
            }, 1);
        }
    }
});

/**
 * If true, you will lose hearts when dying from natural sources (Default: True)
 * Can be toggled with ?envsteal <true/false>
 */
let envSteal: boolean;
system.runInterval(() => {
    if (lifesteal.get(`envsteal`) == `true`) envSteal = true;
    if (lifesteal.get(`envsteal`) == `false`) envSteal = false;
}, 1);

/**
 * Lifesteal
 */

system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        try {
            if (getScore(player, `lsh-give`) > 0) {
                player.runCommandAsync(`scoreboard players remove @s lsh-give 1`);
                return;
            }
        } catch {
            player.runCommandAsync(`scoreboard players set @s lsh-give -1`);
            return;
        }
        player.runCommandAsync(`scoreboard players set @s lsh-give -1`);
        player.getTags().forEach(x => {
            if (x.startsWith(`lsh-`)) {
                player.removeTag(x);
            }
        });
    }
}, 20);

world.events.entityHurt.subscribe(data => {
    let player = data.hurtEntity as Player;
    let damagingEntity = data.damageSource.damagingEntity as Player;

    if (data.damageSource.damagingEntity) {
        if (damagingEntity.typeId !== `minecraft:player`) return;
        if (player.getTags()) {
            player.getTags().forEach(x => {
                if (x.startsWith(`lsh-`)) {
                    player.removeTag(x);
                }
            });
        }
        player.addTag(`lsh-${damagingEntity.name}`);
        player.runCommandAsync(`scoreboard players set @s lsh-give 30`);
    }
});

world.events.entityDie.subscribe(data => {
    let player = data.deadEntity as Player;
    let target = data.damageSource.damagingEntity as Player;
    let health = player.getComponent(`health`) as EntityHealthComponent;
    if (!target) {
        if (player.getTags()) {
            player.getTags().forEach(x => {
                if (getPlayer(x.slice(4)) && x.startsWith(`lsh-`)) target = getPlayer(x.slice(4));
            });
        }
    }
    player.getTags().forEach(x => {
        if (x.startsWith(`lsh-`)) {
            player.removeTag(x);
            player.runCommandAsync(`scoreboard players set @s lsh-give -1`);
        }
    });

    if (player.typeId !== `minecraft:player`) return;

    if (envSteal == true) {
        if (!player) return;

        lifestealLogs.push(`§6${time} > ${player.name} lost ${+lifesteal.get(`heartgl`)} heart(s)`);
        player.runCommandAsync(`scoreboard players remove @s ls-death ${+lifesteal.get(`heartgl`)}`);

        if (!target) return;
        if (target.typeId != `minecraft:player`) return;

        if (health.current <= 0) target.runCommandAsync(`scoreboard players add @s ls-death ${+lifesteal.get(`heartgl`)}`);
        lifestealLogs.push(`§6${time} > ${target.name} gained ${+lifesteal.get(`heartgl`)} heart(s) from killing ${player.name}`);
    }
    if (envSteal == false) {
        if (!player || !target) return;
        if (target.typeId != `minecraft:player`) return;

        target.runCommandAsync(`scoreboard players add @s ls-death ${+lifesteal.get(`heartgl`)}`);
        player.runCommandAsync(`scoreboard players remove @s ls-death ${+lifesteal.get(`heartgl`)}`);
        lifestealLogs.push(`§6${time} > ${player.name} lost ${+lifesteal.get(`heartgl`)} heart(s)`);
        lifestealLogs.push(`§6${time} > ${target.name} gained ${+lifesteal.get(`heartgl`)} heart(s) from killing ${player.name}`);
    }
});

// Item lore
system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        let inv = player.getComponent(`inventory`) as EntityInventoryComponent;
        for (let i = 0; i < inv.container.size; i++) {
            let it = inv.container.getItem(i);
            if (!it) continue;
            if (it.typeId == `lifesteal:heart_of_life`) {
                if (it.getLore()[0] == `§r§7This is the heart of life.\nUse this to revive a player.\n\n(( §o§fRight click/eat or do ?revive§7 to use ))`) continue;
                let item = new ItemStack(ItemTypes.get(`lifesteal:heart_of_life`), it.amount);
                item.setLore([`§r§7This is the heart of life.\nUse this to revive a player.\n\n(( §o§fRight click/eat or do ?revive§7 to use§r§7 ))`]);
                inv.container.setItem(i, item);
            }
            if (it.typeId == `lifesteal:heart_fragment`) {
                if (it.getLore()[0] == `§r§dUse 4 of these to craft a Heart.`) continue;
                let item = new ItemStack(ItemTypes.get(`lifesteal:heart_fragment`), it.amount);
                item.setLore([`§r§dUse 4 of these to craft a Heart.`]);
                inv.container.setItem(i, item);
            }
            if (it.typeId == `lifesteal:heart`) {
                if (it.getLore()[0] == `§r§7A heart is used to add a heart to your health.\n\n(( §o§fRight click/eat§7 to recieve a heart ))`) continue;
                let item = new ItemStack(ItemTypes.get(`lifesteal:heart`), it.amount);
                item.setLore([`§r§7A heart is used to add a heart to your health.\n\n(( §o§fRight click/eat§7 to recieve a heart ))`]);
                inv.container.setItem(i, item);
            }
            if (it.typeId == `lifesteal:heart_essence`) {
                if (it.getLore()[0] == `§r§dUse this to craft a Heart.`) continue;
                let item = new ItemStack(ItemTypes.get(`lifesteal:heart_essence`), it.amount);
                item.setLore([`§r§dUse this to craft a Heart.`]);
                inv.container.setItem(i, item);
            }
        }
    }
}, 1);

// Dead and revive system
system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        if (player.hasTag(`ls-dead`) && !deadplayers.has(player.name)) {
            if (!revivelist.has(player.name)) deadplayers.set(`${player.name}`, `${player.name}`);
        }
        if (deadplayers.has(player.name) && !player.hasTag(`ls-dead`)) {
            deadplayers.delete(`${player.name}`);
        }
    }
}, 1);

system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        if (revivelist.has(player.name.toLowerCase())) {
            player.runCommandAsync(`tag @s add ls-revive`);
            revivelist.delete(player.name.toLowerCase());
        }
    }
}, 1);